﻿/*
 *
 *
 *    BipolarCounter.cs
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2014 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *    
 *
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace KK5JY.AutoCap {
	/// <summary>
	/// Tracks a long-term bipolar (three-state) signal.
	/// </summary>
	public class BipolarCounter {
		#region Private Fields
		private ulong m_Count;
		private long m_Value;
		private int m_LastInput;
		#endregion

		#region Public Properties
		/// <summary>
		/// The absolute counter limit.
		/// </summary>
		public int Limit { get; set; }

		/// <summary>
		/// Returns the net sign of the counter.
		/// </summary>
		public int Sign {
			get {
				if (m_Value < 0) return -1;
				if (m_Value > 0) return 1;
				return 0;
			}
		}

		/// <summary>
		/// Returns the net sign of the counter,
		/// or the most recent value if there is a tie.
		/// </summary>
		public int BinarySign {
			get {
				var result = Sign;
				if (result == 0) {
					if (m_LastInput != 0) {
						return m_LastInput; // return the last input
					} else {
						return 1; // give up, default sign is positive
					}
				}
				// return the real net sign
				return result;
			}
		}

		/// <summary>
		/// Returns the net value of the counter.
		/// </summary>
		public long Value {
			get { return m_Value; }
		}

		/// <summary>
		/// Returns the last input.
		/// </summary>
		public int LastInput {
			get { return m_LastInput; }
		}
		#endregion

		#region Public Methods
		/// <summary>
		/// Initialize a new counter.
		/// </summary>
		public BipolarCounter() {
			Reset();
			Limit = 3;
		}

		/// <summary>
		/// Reset the counter.
		/// </summary>
		/// <param name="value">The first initial value.</param>
		public void Reset(int value = 0) {
			if (value == 0) {
				m_Value = 0;
				m_Count = 0UL;
				m_LastInput = 0;
			} else {
				m_Count = 1UL;

				if (m_Value > Limit) m_Value = Limit;
				else if (m_Value < -Limit) m_Value = -Limit;
				else m_Value = value;

				if (value > 0) m_LastInput = 1;
				else if (value < 0) m_LastInput = -1;
				else m_LastInput = 0;
			}
		}

		/// <summary>
		/// Update the counter.
		/// </summary>
		/// <param name="value">The new value.  Only the sign is considered.</param>
		public void Update(decimal value) {
			if (value < 0) {
				Down();
			} else if (value > 0) {
				Up();
			} else {
				Hold();
			}
		}

		/// <summary>
		/// Update the counter.
		/// </summary>
		/// <param name="value">The new value.  Only the sign is considered.</param>
		public void Update(double value) {
			if (value < 0) {
				Down();
			} else if (value > 0) {
				Up();
			} else {
				Hold();
			}
		}

		/// <summary>
		/// Update the counter.
		/// </summary>
		/// <param name="value">The new value.  Only the sign is considered.</param>
		public void Update(float value) {
			if (value < 0) {
				Down();
			} else if (value > 0) {
				Up();
			} else {
				Hold();
			}
		}

		/// <summary>
		/// Update the counter.
		/// </summary>
		/// <param name="value">The new value.  Only the sign is considered.</param>
		public void Update(int value) {
			if (value < 0) {
				Down();
			} else if (value > 0) {
				Up();
			} else {
				Hold();
			}
		}
		#endregion

		#region Utilities
		/// <summary>
		/// Tick up.
		/// </summary>
		private void Up() {
			if (m_Value < Limit)
				++m_Value;
			++m_Count;
			m_LastInput = 1;
		}

		/// <summary>
		/// Tick down.
		/// </summary>
		private void Down() {
			if (m_Value > -Limit)
				--m_Value;
			++m_Count;
			m_LastInput = -1;
		}

		/// <summary>
		/// Don't change the net value, but store last input and count.
		/// </summary>
		private void Hold() {
			++m_Count;
			m_LastInput = 0;
		}
		#endregion
	}
}