﻿/*
 *
 *
 *    HRD.cs
 *
 *    HRD 5.0 interface.
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2014,2016 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *    
 *
 */

using System;
using System.Net.Sockets;
using System.Text;
using System.Diagnostics;

namespace KK5JY.RadioSupport {
	/// <summary>
	/// Basic HRD interface.
	/// </summary>
	public class HRD : IDisposable {
		#region Statics
		private static byte[] ProductQuery = ConvertQueryString(
			"22 00 00 00 cd ab 34 12 34 12 cd ab 00 00 00 00 " +
			"47 00 65 00 74 00 20 00 49 00 44 00 00 00 00 00 " +
			"00 00"
		);

		private static byte[] VersionQuery = ConvertQueryString(
			"2c 00 00 00 cd ab 34 12 34 12 cd ab 00 00 00 00 " +
			"47 00 65 00 74 00 20 00 56 00 65 00 72 00 73 00 " +
			"69 00 6f 00 6e 00 00 00 00 00 00 00"
		);

		private static byte[] RadioQuery = ConvertQueryString(
			"2a 00 00 00 cd ab 34 12 34 12 cd ab 00 00 00 00 " +
			"47 00 65 00 74 00 20 00 52 00 61 00 64 00 69 00 " +
			"6f 00 73 00 00 00 00 00 00 00"
		);

        private static byte[] FrequencyQueryHead = ConvertQueryString(
			"38 00 00 00 cd ab 34 12 34 12 cd ab 00 00 00 00 " +
			"5b 00"
        );
        private static byte[] FrequencyQueryTail = ConvertQueryString(
                        "5d 00 20 00 47 00 65 00 74 00 20 00 " +
			"46 00 72 00 65 00 71 00 75 00 65 00 6e 00 63 00 " +
			"79 00 00 00 00 00 00 00"
        );
        private /*not static*/ byte[] FrequencyQuery = ConvertQueryString(
			"38 00 00 00 cd ab 34 12 34 12 cd ab 00 00 00 00 " +
			"5b 00 31 00 5d 00 20 00 47 00 65 00 74 00 20 00 " +
			"46 00 72 00 65 00 71 00 75 00 65 00 6e 00 63 00 " +
			"79 00 00 00 00 00 00 00"
		);

        private void RebuildFrequencyQuery()
        {
            if (m_RadioIndex < 0) return;
            var radioChars = m_RadioIndex.ToString();
            FrequencyQuery = new byte[FrequencyQueryHead.Length + FrequencyQueryTail.Length + (2 * radioChars.Length)];
            FrequencyQueryHead.CopyTo(FrequencyQuery, 0);
            for (int i = 0; i != radioChars.Length; ++i)
            {
                FrequencyQuery[FrequencyQueryHead.Length + (2 * i)] = (byte)(radioChars[i]);
                FrequencyQuery[FrequencyQueryHead.Length + (2 * i) + 1] = 0;
            }
            FrequencyQueryTail.CopyTo(FrequencyQuery, FrequencyQueryHead.Length + (2 * radioChars.Length));
            FrequencyQuery[0] = Convert.ToByte(FrequencyQuery.Length); // Convert.ToByte(0x36 + (2 * radioChars.Length));
        }

        private static byte[] ConvertQueryString(string p)
        {
			var tokens = p.Split(new[] { " " }, StringSplitOptions.RemoveEmptyEntries);
			var result = new byte[tokens.Length];
			int i = 0;
			foreach (var token in tokens) {
				result[i++] = (byte)Convert.ToInt32(token, 16);
			}
			return result;
		}

        private string ConvertQueryToString(byte[] data)
        {
            var result = new StringBuilder();
            foreach (var b in data)
            {
                if (b != 0)
                {
                    result.Append((char)(b));
                }
            }
            return result.ToString();
        }
		#endregion

		#region Public Properties
		/// <summary>
		/// Fetch the product name.
		/// </summary>
		public string Product { get { return ExecuteQuery(ProductQuery); } }

		/// <summary>
		/// Fetch the version string.
		/// </summary>
		public string Version { get { return ExecuteQuery(VersionQuery); } }

		/// <summary>
		/// Fetch the radio name.
		/// </summary>
		public string Radio { get { return ExecuteQuery(RadioQuery); } }

		/// <summary>
		/// Fetch the frequency.
		/// </summary>
		public int Frequency {
            get
            {
                if (m_RadioIndex < 0)
                {
                    var radio = Radio;

                    // have to read the radio, or we can't read frequency
                    if (String.IsNullOrEmpty(radio))
                    {
                        return 0;
                    }

                    // try to parse the radio index
                    var parts = radio.Split(':');
                    if (parts.Length != 2)
                    {
                        return 0;
                    }
                    if (!Int32.TryParse(parts[0], out m_RadioIndex))
                    {
                        return 0;
                    }
                    if (m_RadioIndex < 0)
                    {
                        return 0;
                    }

                    // now form the proper frequency query
                    Debug.WriteLine(String.Format("Frequency Old: \"{0}\"", ConvertQueryToString(FrequencyQuery)));
                    RebuildFrequencyQuery();
                    Debug.WriteLine(String.Format("Frequency New: \"{0}\"", ConvertQueryToString(FrequencyQuery)));
                }
                var hrdstr = ExecuteQuery(FrequencyQuery);
                Int32 result;
                if (!Int32.TryParse(hrdstr, out result))
                {
                    // if this failed, the radio index may have changed
                    m_RadioIndex = -1;
                    return 0;
                }
                return result;
            }
        }
		#endregion

		#region Private Fields
		private TcpClient m_Client;
		private NetworkStream m_Stream;
		private byte[] m_InBuffer;
        private int m_RadioIndex;
		#endregion

		#region Public Methods
		/// <summary>
		/// Create a new HRD object.
		/// </summary>
		public HRD() {
			m_InBuffer = new byte[1024];
            m_RadioIndex = -1;

            // DEBUG:
            Debug.WriteLine(String.Format("Product  : \"{0}\"", ConvertQueryToString(ProductQuery)));
            Debug.WriteLine(String.Format("Version  : \"{0}\"", ConvertQueryToString(VersionQuery)));
            Debug.WriteLine(String.Format("Radio    : \"{0}\"", ConvertQueryToString(RadioQuery)));
        }

		/// <summary>
		/// Open a connection to HRD.
		/// </summary>
		/// <param name="host">The hostname.</param>
		/// <param name="port">The port number (TCP).</param>
		public void Open(string host, int port = 7809) {
			if (m_Stream != null)
				throw new ApplicationException("Connection already open");
			m_Client = new TcpClient(host, port);
			m_Stream = m_Client.GetStream();
		}

		/// <summary>
		/// Close the connection.
		/// </summary>
		public void Close() {
			try { m_Stream.Dispose(); } catch { /* nop */ }
			try { m_Client.Close(); } catch { /* nop */ }
			m_Stream = null;
			m_Client = null;
		}

		/// <summary>
		/// Synonym for Close().
		/// </summary>
		public void Dispose() {
			Close();
		}
		#endregion

		#region Utilities
		private string ExecuteQuery(byte[] queryData) {
			m_Stream.Write(queryData, 0, queryData.Length);
			m_Stream.Flush();

			m_InBuffer[0] = (byte)m_Stream.ReadByte();
			var count = m_Stream.Read(m_InBuffer, 1, m_InBuffer[0]);

			string result = "";
			for (int i = 0x10; i < count; i += 2) {
				if (m_InBuffer[i] != 0)
					result += (char)m_InBuffer[i];
			}
			return result;
		}
		#endregion
	}
}
