﻿/*
 *
 *
 *    PortUtils.cs
 *
 *    Device and network address management.
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2014 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *    
 *
 */

using System;
using System.Net;
using System.IO.Ports;

namespace KK5JY.AutoCap {
    /// <summary>
    /// Assorted utilities for configuring port names.
    /// </summary>
    public static class PortUtils {
        /// <summary>
        /// Parser for endpoint strings.
        /// </summary>
        /// <param name="s">The input string.</param>
        /// <param name="ep">The resulting endpoint.</param>
        /// <returns>True if successful.</returns>
        public static bool TryParse(string s, out IPEndPoint ep) {
            ep = null;

            var parts = s.Split(':');
            if (parts.Length != 2) {
                return false;
            }
            int port = 0;
            if (!Int32.TryParse(parts[1], out port)) {
                return false;
            }
            if (port < 0 || port > UInt16.MaxValue) {
                return false;
            }
            IPAddress ip;
            if (IPAddress.TryParse(parts[0], out ip)) {
                ep = new IPEndPoint(ip, port);
                return true;
            }
            try {
                var ips = Dns.GetHostEntry(parts[0]);
                if (ips == null || ips.AddressList.Length == 0) {
                    return false;
                }
                ep = new IPEndPoint(ips.AddressList[0], port);
                return true;
            } catch {
                return false;
            }
        }

        /// <summary>
        /// Check for valid serial device.
        /// </summary>
        /// <param name="s">The serial port name.</param>
        /// <returns>True if successful.</returns>
        public static bool PortExists(string s) {
#if __MonoCS__
			// special case for Arduino serial devices
			if (s.StartsWith("/dev/ttyACM", StringComparison.InvariantCultureIgnoreCase) && System.IO.File.Exists(s)) {
				return true;
			}
#endif

            foreach (var dev in SerialPort.GetPortNames()) {
                if (s.StartsWith(dev, StringComparison.InvariantCultureIgnoreCase)) {
                    return true;
                }
            }

            return false;
        }
    }
}
