﻿/*
 *
 *
 *    SWR.cs
 *
 *    SWR Calculations.
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2014 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *    
 *
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Numerics;

namespace KK5JY.AutoCap {
	/// <summary>
	/// SWR Calculator.
	/// </summary>
	public static class SWR {
		/// <summary>
		/// Compute SWR from two complex impedence values.
		/// </summary>
		/// <param name="Zo">The characteristic impedence of the system.</param>
		/// <param name="Zl">The load impedence.</param>
		/// <returns>The computed VSWR.</returns>
		public static double ComputeSWR(Complex Zo, Complex Zl) {
			var rho = Complex.Abs((Zl - Zo) / (Zl + Zo));
			return (1 + rho) / (1 - rho);
		}

		/// <summary>
		/// Compute SWR from real values and load phase angle.
		/// </summary>
		/// <param name="Zo">The characteristic impedence of the system.</param>
		/// <param name="R">The load impedence, real part.</param>
		/// <param name="Phi">The load phase angle, in degrees.</param>
		/// <returns>The computed VSWR.</returns>
		public static double ComputeSWR(double Zo, double R, double Phi) {
			return ComputeSWR(new Complex(Zo, 0), Complex.FromPolarCoordinates(R, DegreeToRadian(Phi)));
		}

		/// <summary>
		/// Convert degrees to radians.
		/// </summary>
		private static double DegreeToRadian(double angleInDegrees) {
			return Math.PI * angleInDegrees / 180.0;
		}
	}
}
